;  48 bit display with 2 colors
;  By Omegamatrix
;
;  This program is is inspired by Spiceware's 48 bit display with 2 colors found here:
;
;  http://www.atariage.com/forums/topic/197100-asm-or-bbasic/#entry2519749
;
;
;  In this program uses a un-rolled display loop with a lot of absolute,Y indexing.
;  Although expensive in rom space, it does allow the user to create a 2 color
;  48 byte display with the second color starting at any pixel, and the background
;  does not have to be black.
;
;  The routine can be modified to be re-usable. By loading three of the digits into ram.
;  This could be possible in a title screen where not much ram is required. Repositioning
;  would have to be done mid-kernel with a 73 or 74 cycle Hmove to hide the Hmove lines.
;  An example of repositioning at cycle 74 (always taking exactly 1 line!) can be found here:
;
;  http://www.atariage.com/forums/topic/183219-cycle-74-hmoves/
;


      processor 6502

VSYNC   =  $00
VBLANK  =  $01
WSYNC   =  $02
NUSIZ0  =  $04
NUSIZ1  =  $05
COLUP0  =  $06
COLUP1  =  $07
COLUPF  =  $08
COLUBK  =  $09
CTRLPF  =  $0A
PF0     =  $0D
PF1     =  $0E
PF2     =  $0F
RESP0   =  $10
RESP1   =  $11
RESM0   =  $12
RESM1   =  $13
GRP0    =  $1B
GRP1    =  $1C
ENAM0   =  $1D
ENAM1   =  $1E
ENABL   =  $1F
HMP0    =  $20
HMP1    =  $21
HMM0    =  $22
HMM1    =  $23
VDELP0  =  $25
VDELP1  =  $26
HMOVE   =  $2A
HMCLR   =  $2B

SWCHA   =  $0280
INTIM   =  $0284
TIM64T  =  $0296


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      RAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG.U Variables
       ORG $80


gfx_PF1          ds 1
gfx_PF0          ds 1
gfx_PF2          ds 1
gfxPtrs          ds 12
frameCounter     ds 1

ballPos          ds 1
tempOne          ds 1


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      USER CONSTANTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;HMPx
LEFT_7          = $70
LEFT_6          = $60
LEFT_5          = $50
LEFT_4          = $40
LEFT_3          = $30
LEFT_2          = $20
LEFT_1          = $10
NO_MOTION       = $00
RIGHT_1         = $F0
RIGHT_2         = $E0
RIGHT_3         = $D0
RIGHT_4         = $C0
RIGHT_5         = $B0
RIGHT_6         = $A0
RIGHT_7         = $90
RIGHT_8         = $80

COL_LEFT_SIDE   = $1A
COL_RIGHT_SIDE  = $C6
COL_BACKGROUND  = $A0

RIGHT_EDGE = 104
LEFT_EDGE  = 56

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      MAIN PROGRAM
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG Code
       ORG $F000

START:
    sei
    cld
    ldx    #0
    txa
.loopClear:
    dex
    txs
    pha
    bne    .loopClear

;initialization
    sta   WSYNC
;---------------------------------------
    lda    #$33                  ;2  @2
    sta    NUSIZ0                ;3  @5
    sta    NUSIZ1                ;3  @8
    sta    VDELP0                ;3  @11
    sta    VDELP1                ;3  @14
    lda    #COL_BACKGROUND       ;2  @16
    sta    COLUP0                ;3  @19
    sta    COLUP1                ;3  @22
    nop                          ;2  @24
    nop                          ;2  @26
    sta    RESM0                 ;3  @29
    lda    #LEFT_7               ;2  @31
    sta    HMM0                  ;3  @34
    sta    RESP0                 ;3  @37
    sta    RESP1                 ;3  @40
    lda    #RIGHT_8              ;2  @42
    sta    HMP0                  ;3  @45
    lda    #RIGHT_7              ;2  @47
    sta    HMP1                  ;3  @50
    lda    #LEFT_6               ;2  @52
    sta    HMM1                  ;3  @55
    sta    RESM1                 ;3  @58


    sta    WSYNC
;---------------------------------------
    sta    HMOVE

    lda    #>ColorOne
    sta    gfxPtrs+1
    sta    gfxPtrs+3
    sta    gfxPtrs+5
    sta    gfxPtrs+7
    sta    gfxPtrs+9
    sta    gfxPtrs+11

    lda    #<ColorOne
    sta    gfxPtrs
    lda    #<ColorTwo
    sta    gfxPtrs+2
    lda    #<ColorThree
    sta    gfxPtrs+4
    lda    #<ColorFour
    sta    gfxPtrs+6
    lda    #<ColorFive
    sta    gfxPtrs+8
    lda    #<ColorSix
    sta    gfxPtrs+10

    lda    #RIGHT_EDGE
    sta    ballPos
    sta    HMCLR


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      Main Loop
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MainLoop:
    lda    #$0E
.loopVsync:
    sta    WSYNC
;---------------------------------------
    sta    VSYNC
    lsr
    bne    .loopVsync
    lda    #50
    sta    TIM64T


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  check joystick, update ball Hpos
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    lda    frameCounter
    and    #$07
    bne    .exitJoystick
    ldx    ballPos
    lda    SWCHA
;check Right
    asl
    bcs    .checkLeft
    inx
    cpx    #RIGHT_EDGE+1
    bcc    .storeBallPos1
    dex
.storeBallPos1:
    stx    ballPos
.checkLeft:
    asl
    bcs    .exitJoystick
    dex
    cpx    #LEFT_EDGE
    bcs    .storeBallPos2
    inx
.storeBallPos2:
    stx    ballPos
.exitJoystick:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  position ball
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldx    #4
    lda    ballPos
    clc
    adc    #1       ; correction factor, good for middle range positioning
    sec
    sta    WSYNC
;---------------------------------------
.divide15:
    sbc    #15
    bcs    .divide15
    eor    #7
    asl
    asl
    asl
    asl
    sta.wx HMP0,X
    sta    RESP0,X

    sta    WSYNC
;---------------------------------------
    sta    HMOVE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  find playfield shape
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    lda    ballPos
    lsr
    lsr
    sec
    sbc    #14
    tay
    lda   PF1_Tab,Y
    sta   gfx_PF1
    lda   PF0_Tab,Y
    sta   gfx_PF0
    lda   PF2_Tab,Y
    sta   gfx_PF2

.waitVblank:
    lda    INTIM
    bne    .waitVblank
    sta    WSYNC
;---------------------------------------
    sta    VBLANK                   ; Vblank ends






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  top blank space
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldy    #81
.loopTop:
    sta    WSYNC
;---------------------------------------
    dey                          ;2  @2
    bpl    .loopTop              ;2 @4/5

    dec    $2E                   ;5  @9
    dec    $2E                   ;5  @14
    dec    $2E                   ;5  @19
    dec    $2E                   ;5  @24
    dec    $2E                   ;5  @29
    dec    $2E                   ;5  @34
    dec    $2E                   ;5  @39
    dec    $2E                   ;5  @44
    dec    $2E                   ;5  @49
    dec    $2E                   ;5  @54
    nop                          ;2  @56
    nop                          ;2  @58
    nop                          ;2  @60
    nop                          ;2  @62


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  demo bar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldy    #7                    ;2  @64
    sty    ENAM0                 ;3  @67
    sty    ENABL                 ;3  @70
    sty    ENAM1                 ;3  @73

.loopDemoBar:
    sta    WSYNC
;---------------------------------------
    lda    gfx_PF0               ;3  @3
    sta    PF0                   ;3  @6
    lda    gfx_PF1               ;3  @9
    sta    PF1                   ;3  @12
    lda    gfx_PF2               ;3  @15
    sta    PF2                   ;3  @18
    lda    #$20                  ;2  @20
    sta.w  CTRLPF                ;4  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    dec    $2E                   ;5  @34
    lda    #COL_LEFT_SIDE        ;2  @36
    sta    COLUBK                ;3  @39
    dec    $2E                   ;5  @44
    dec    $2E                   ;5  @49
    nop                          ;2  @51
    nop                          ;2  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    dey                          ;2  @63
    bpl  .loopDemoBar            ;2 @65/66


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  48 bit display
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    sta    WSYNC
;---------------------------------------
    ldy    #7
    lda    (gfxPtrs+10),Y
    sta    tempOne
    ldx    ColorFour,Y
    txs

    sta    WSYNC
;---------------------------------------
    lda    ColorOne,Y            ;4  @4
    sta.w  GRP0                  ;4  @8
    lda    ColorTwo,Y            ;4  @12
    sta    GRP1                  ;3  @15
    lda    ColorThree,Y          ;4  @19
    sta    GRP0                  ;3  @22
    ldx    #COL_LEFT_SIDE        ;2  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    lda    ColorFive,Y           ;4  @33
    ldy    tempOne               ;3  @36
    stx    COLUBK                ;3  @39
    tsx                          ;2  @41
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    ldy    #6                    ;2  @63
    lda    (gfxPtrs+10),Y        ;5  @68
    sta    tempOne               ;3  @71
    ldx    ColorFour,Y           ;4  @75
;---------------------------------------
    txs                          ;2  @1
    lda    ColorOne,Y            ;4  @5
    sta    GRP0                  ;3  @8
    lda    ColorTwo,Y            ;4  @12
    sta    GRP1                  ;3  @15
    lda    ColorThree,Y          ;4  @19
    sta    GRP0                  ;3  @22
    ldx    #COL_LEFT_SIDE        ;2  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    lda    ColorFive,Y           ;4  @33
    ldy    tempOne               ;3  @36
    stx    COLUBK                ;3  @39
    tsx                          ;2  @41
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    ldy    #5                    ;2  @63
    lda    (gfxPtrs+10),Y        ;5  @68
    sta    tempOne               ;3  @71
    ldx    ColorFour,Y           ;4  @75
;---------------------------------------
    txs                          ;2  @1
    lda    ColorOne,Y            ;4  @5
    sta    GRP0                  ;3  @8
    lda    ColorTwo,Y            ;4  @12
    sta    GRP1                  ;3  @15
    lda    ColorThree,Y          ;4  @19
    sta    GRP0                  ;3  @22
    ldx    #COL_LEFT_SIDE        ;2  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    lda    ColorFive,Y           ;4  @33
    ldy    tempOne               ;3  @36
    stx    COLUBK                ;3  @39
    tsx                          ;2  @41
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    ldy    #4                    ;2  @63
    lda    (gfxPtrs+10),Y        ;5  @68
    sta    tempOne               ;3  @71
    ldx    ColorFour,Y           ;4  @75
;---------------------------------------
    txs                          ;2  @1
    lda    ColorOne,Y            ;4  @5
    sta    GRP0                  ;3  @8
    lda    ColorTwo,Y            ;4  @12
    sta    GRP1                  ;3  @15
    lda    ColorThree,Y          ;4  @19
    sta    GRP0                  ;3  @22
    ldx    #COL_LEFT_SIDE        ;2  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    lda    ColorFive,Y           ;4  @33
    ldy    tempOne               ;3  @36
    stx    COLUBK                ;3  @39
    tsx                          ;2  @41
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    ldy    #3                    ;2  @63
    lda    (gfxPtrs+10),Y        ;5  @68
    sta    tempOne               ;3  @71
    ldx    ColorFour,Y           ;4  @75
;---------------------------------------
    txs                          ;2  @1
    lda    ColorOne,Y            ;4  @5
    sta    GRP0                  ;3  @8
    lda    ColorTwo,Y            ;4  @12
    sta    GRP1                  ;3  @15
    lda    ColorThree,Y          ;4  @19
    sta    GRP0                  ;3  @22
    ldx    #COL_LEFT_SIDE        ;2  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    lda    ColorFive,Y           ;4  @33
    ldy    tempOne               ;3  @36
    stx    COLUBK                ;3  @39
    tsx                          ;2  @41
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    ldy    #2                    ;2  @63
    lda    (gfxPtrs+10),Y        ;5  @68
    sta    tempOne               ;3  @71
    ldx    ColorFour,Y           ;4  @75
;---------------------------------------
    txs                          ;2  @1
    lda    ColorOne,Y            ;4  @5
    sta    GRP0                  ;3  @8
    lda    ColorTwo,Y            ;4  @12
    sta    GRP1                  ;3  @15
    lda    ColorThree,Y          ;4  @19
    sta    GRP0                  ;3  @22
    ldx    #COL_LEFT_SIDE        ;2  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    lda    ColorFive,Y           ;4  @33
    ldy    tempOne               ;3  @36
    stx    COLUBK                ;3  @39
    tsx                          ;2  @41
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    ldy    #1                    ;2  @63
    lda    (gfxPtrs+10),Y        ;5  @68
    sta    tempOne               ;3  @71
    ldx    ColorFour,Y           ;4  @75
;---------------------------------------
    txs                          ;2  @1
    lda    ColorOne,Y            ;4  @5
    sta    GRP0                  ;3  @8
    lda    ColorTwo,Y            ;4  @12
    sta    GRP1                  ;3  @15
    lda    ColorThree,Y          ;4  @19
    sta    GRP0                  ;3  @22
    ldx    #COL_LEFT_SIDE        ;2  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    lda    ColorFive,Y           ;4  @33
    ldy    tempOne               ;3  @36
    stx    COLUBK                ;3  @39
    tsx                          ;2  @41
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    ldy    #0                    ;2  @63
    lda    (gfxPtrs+10),Y        ;5  @68
    sta    tempOne               ;3  @71
    ldx    ColorFour,Y           ;4  @75
;---------------------------------------
    txs                          ;2  @1
    lda    ColorOne,Y            ;4  @5
    sta    GRP0                  ;3  @8
    lda    ColorTwo,Y            ;4  @12
    sta    GRP1                  ;3  @15
    lda    ColorThree,Y          ;4  @19
    sta    GRP0                  ;3  @22
    ldx    #COL_LEFT_SIDE        ;2  @24
    lda    #COL_RIGHT_SIDE       ;2  @26
    sta    COLUPF                ;3  @29
    lda    ColorFive,Y           ;4  @33
    ldy    tempOne               ;3  @36
    stx    COLUBK                ;3  @39
    tsx                          ;2  @41
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    lda    #COL_BACKGROUND       ;2  @55
    sta    COLUBK                ;3  @58
    sta    COLUPF                ;3  @61
    lda    #0                    ;2  @63
    sta    GRP0                  ;3  @66
    sta    GRP1                  ;3  @69
    sta    GRP0                  ;3  @72
    sta    GRP1                  ;3  @75
;---------------------------------------
    sta    ENAM0                 ;3  @2
    sta    ENAM1
    sta    ENABL
    sta    PF0
    sta    PF1
    sta    PF2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  bottom blank space
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldy    #92
.loopBottom:
    sta    WSYNC
;---------------------------------------
    dey
    bne    .loopBottom

    sta    WSYNC
;---------------------------------------
    lda    #2
    sta    VBLANK
    lda    #28
    sta    TIM64T

.waitOverscan:
    lda    INTIM
    bne    .waitOverscan
    jmp    MainLoop




       ORG $FF00

ColorOne:
    .byte $FF ; |XXXXXXXX| $FF00
    .byte $C3 ; |XX    XX| $FF01
    .byte $9D ; |X  XXX X| $FF02
    .byte $9F ; |X  XXXXX| $FF03
    .byte $9F ; |X  XXXXX| $FF04
    .byte $9F ; |X  XXXXX| $FF05
    .byte $9D ; |X  XXX X| $FF06
    .byte $C3 ; |XX    XX| $FF07
ColorTwo:
    .byte $FF ; |XXXXXXXX| $FF08
    .byte $C3 ; |XX    XX| $FF09
    .byte $99 ; |X  XX  X| $FF0A
    .byte $99 ; |X  XX  X| $FF0B
    .byte $99 ; |X  XX  X| $FF0C
    .byte $99 ; |X  XX  X| $FF0D
    .byte $99 ; |X  XX  X| $FF0E
    .byte $C3 ; |XX    XX| $FF0F
ColorThree:
    .byte $FF ; |XXXXXXXX| $FF10
    .byte $81 ; |X      X| $FF11
    .byte $9F ; |X  XXXXX| $FF12
    .byte $9F ; |X  XXXXX| $FF13
    .byte $9F ; |X  XXXXX| $FF14
    .byte $9F ; |X  XXXXX| $FF15
    .byte $9F ; |X  XXXXX| $FF16
    .byte $9F ; |X  XXXXX| $FF17
ColorFour:
    .byte $FF ; |XXXXXXXX| $FF18
    .byte $C3 ; |XX    XX| $FF19
    .byte $99 ; |X  XX  X| $FF1A
    .byte $99 ; |X  XX  X| $FF1B
    .byte $99 ; |X  XX  X| $FF1C
    .byte $99 ; |X  XX  X| $FF1D
    .byte $99 ; |X  XX  X| $FF1E
    .byte $C3 ; |XX    XX| $FF1F
ColorFive:
    .byte $FF ; |XXXXXXXX| $FF20
    .byte $99 ; |X  XX  X| $FF21
    .byte $99 ; |X  XX  X| $FF22
    .byte $99 ; |X  XX  X| $FF23
    .byte $87 ; |X    XXX| $FF24
    .byte $99 ; |X  XX  X| $FF25
    .byte $99 ; |X  XX  X| $FF26
    .byte $87 ; |X    XXX| $FF27
ColorSix:
    .byte $FF ; |XXXXXXXX| $FF28
    .byte $83 ; |X     XX| $FF29
    .byte $F9 ; |XXXXX  X| $FF2A
    .byte $F9 ; |XXXXX  X| $FF2B
    .byte $C3 ; |XX    XX| $FF2C
    .byte $9F ; |X  XXXXX| $FF2D
    .byte $9F ; |X  XXXXX| $FF2E
    .byte $C1 ; |XX     X| $FF2F

PF1_Tab:
    .byte $C0
    .byte $C0
    .byte $C0
    .byte $C0
    .byte $C0
    .byte $C0
    .byte $C0
    .byte $C0
    .byte $C0
    .byte $C0
    .byte $40
    .byte $00
    .byte $00

PF0_Tab:
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $E0
    .byte $C0
    .byte $80
    .byte $00
    .byte $00
    .byte $00
    .byte $00

PF2_Tab:
    .byte $F8
    .byte $F0
    .byte $E0
    .byte $C0
    .byte $80
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

       ORG $FFFC

    .word START
    .word START